// SoundTouch_Handler.cpp

#include "SoundTouch_Handler.h"
#include <iostream>

using namespace std;


// ----------------- Selector ----------------------------------
WindowHandler * selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
   return new SoundTouch_Handler(env, obj);
}

// ----------------- class SoundTouch_Handler --------------------------
// Ctor
SoundTouch_Handler::SoundTouch_Handler(JNIEnv * env, jobject obj) : WindowHandler(env, obj)
{
   pSt = new SoundTouch;
   numChannels = 2;
}

SoundTouch_Handler::~SoundTouch_Handler()
{
   delete pSt;
}

int SoundTouch_Handler::invoke(int val)
{
   if (val == 0)
   {
      const char * versionStr = pSt->getVersionString();
      JNIString(_exposedObj, "versionString").setUTF8(versionStr);
   }

   if (val == 1)
   {
      return (int)pSt->getVersionId();
   }

   if (val == 2)
   {
      float newRate = JNIVar<float>(_exposedObj, "newRate").get();
      pSt->setRate(newRate);
   }

   if (val == 3)
   {
      float newTempo = JNIVar<float>(_exposedObj, "newTempo").get();
      pSt->setTempo(newTempo);
   }

   if (val == 4)
   {
      float newRate = JNIVar<float>(_exposedObj, "newRate").get();
      pSt->setRateChange(newRate);
   }

   if (val == 5)
   {
      float newTempo = JNIVar<float>(_exposedObj, "newTempo").get();
      pSt->setTempoChange(newTempo);
   }

   if (val == 6)
   {
      float newPitch = JNIVar<float>(_exposedObj, "newPitch").get();
      pSt->setPitch(newPitch);
   }

   if (val == 7)
   {
      float newPitch = JNIVar<float>(_exposedObj, "newPitch").get();
      pSt->setPitchOctaves(newPitch);
   }

   if (val == 8)
   {
      int newPitchInt = JNIVar<int>(_exposedObj, "newPitchInt").get();
      pSt->setPitchSemiTones(newPitchInt);
   }

   if (val == 9)
   {
      float newPitch = JNIVar<float>(_exposedObj, "newPitch").get();
      pSt->setPitchSemiTones(newPitch);
   }

   if (val == 10)
   {
      numChannels = JNIVar<int>(_exposedObj, "numChannels").get();
      pSt->setChannels(numChannels);
   }

   if (val == 11)
   {
      int srate = JNIVar<int>(_exposedObj, "srate").get();
      pSt->setSampleRate(srate);
   }

   if (val == 12)
   {
      pSt->flush();
   }

   if (val == 13)
   {
      JNIArray<signed char> jary(_exposedObj, "inBuffer");
      signed char * ary = jary.get();
      int numBytes = JNIVar<int>(_exposedObj, "numBytes").get();
	    int numSamples = numBytes / (2 * numChannels);
	    try
	    {
	       pSt->putSamples((SAMPLETYPE *)ary, numSamples);
	    }
	    catch (runtime_error)
	    {
         JNIMethod<void>(_exposedObj, "throwRuntimeException").call();
	    }
   }

   if (val == 14)
   {
      pSt->clear();
   }

   if (val == 15)
   {
      int settingId = JNIVar<int>(_exposedObj, "settingId").get();
      int value = JNIVar<int>(_exposedObj, "value").get();
      BOOL rc = pSt->setSetting(settingId, value);
	    if (rc)
	       return 1;
	    else
	       return 0;
   }

   if (val == 16)
   {
      int settingId = JNIVar<int>(_exposedObj, "settingId").get();
	    return (int)pSt->getSetting(settingId);
   }

   if (val == 17)
   {
	    return (int)pSt->numUnprocessedSamples();
   }

   if (val == 18)
   {
      JNIArray<signed char> jary(_exposedObj, "outBuffer");
      signed char * ary = jary.get();
      int maxBytes = JNIVar<int>(_exposedObj, "maxBytes").get();
	    int nbBytes =  2 * numChannels * (int)pSt->receiveSamples((SAMPLETYPE *)ary, maxBytes / 4);
	    return nbBytes;
   }

   if (val == 19)
   {
      int maxSamples = JNIVar<int>(_exposedObj, "maxSamples").get();
      return (int)pSt->getOutPipe()->receiveSamples(maxSamples);
   }

   if (val == 20)
   {
      return (int)pSt->getOutPipe()->numSamples();
   }

   if (val == 21)
   {
      return (int)pSt->getOutPipe()->isEmpty();
   }

   return 0;
}

