// JawEx6.java
// Prototype is Acquire1Scan from NIDAQmx distribution
// Acquire analog data and display in modeless dialog
// For demonstration with NI USB-6008 interface
// Apply voltage +-5V max at terminals #2 and #1 (GND) or
// attach 10 kOhm potentiometer at terminals #31/#32
// with potentiometer tap at terminal #2
// Timed by API Sleep

import ch.aplu.jaw.*;
import ch.aplu.util.*;

public class JawEx6 implements ExitListener
{
  // Native variables
  private String channel = "Dev1/ai0";  // DeviceID/channel
  private double min = -10;  // Input range min
  private double max = 10;   // Input range max
  private double value; // Acquired data
  private String errorDescription = ""; // Error report buffer
  private int period = 1000; // Measurement interval (ms)
  // End of native variables

  private NativeHandler nh = 
    new NativeHandler("s:\\myjni\\nidaq\\release\\nidaq.dll",
                      this);  // Provide access by exposing
  private enum State{failed, initializing, running, quitting};
  private volatile State state = State.initializing;

  public JawEx6()
  {
    Console c = Console.init();
    c.addExitListener(this);
    System.out.println("Initializing...");
    System.out.print("   Creating task...");
    if (nh.invoke(0) == -1)
    {
      showError();
      return;
    }
    System.out.println("OK");

    System.out.print("   Open channel...");
    if (nh.invoke(1) == -1)
    {
      showError();
      return;
    }
    System.out.println("OK");

    System.out.print("   Start task...");
    if (nh.invoke(2) == -1)
    {
      showError();
      return;
    }
    System.out.println("OK");

    System.out.println("\nGetting data...");
    int nb = 1;
    state = State.running;
    while (state == State.running)
    {
      if (nh.invoke(3) == -1)
        break;
      else
      {
        System.out.
          println("Data #" + nb + ": " + value + " V");
        nh.invoke(4); // Sleep a while
        nb++;
      }
    }
    if (state == State.running)
    {
      state = State.failed;      
      System.out.println("\nError while acquiring data");
    }
    else
      System.out.println("\nStopped by user");
  }

  private void showError()
  {
    state = State.failed;
    System.out.println("\nError: " + errorDescription);
  }

  public void notifyExit()
  {
    switch (state)
    {
      case failed:
        nh.destroy();
        System.exit(1);
        break;
        
      case initializing:
        // Don't interrupt
        break;

      case running:
        state = State.quitting;
        break;

      case quitting:
        nh.destroy();
        System.exit(0);
        break;
    }
  }
 
  public static void main(String args[])
  {
    new JawEx6();
  }
}
