// JawEx7.java
// ContAcquireNScan from NIDAQmx distribution
// Acquire faster varying analog data and display on GPanel
// For demonstration with NI USB-6008 interface
// connect a first signal at terminals #2 and #32(GND)
// and a second signal at terminals #5 and #32(GND)
// both in range -10..10V

import java.awt.Color;
import ch.aplu.util.*;
import ch.aplu.jaw.*;

public class JawEx7 implements ExitListener
{
  // Transfer variables
  private String channel = "Dev1/ai0, Dev1/ai1"; // AI0 & AI1
  private double min = -10; // Measuring range -10..10V
  private double max = 10;
  private double sampleRate = 5000.0;  // Samples per seconds
  private long samplesPerChannel = 200; // Samples per burst
  private String errorDescription = 
    new String();  //  Verbose error report
  private double[] data = 
    new double[2*(int)samplesPerChannel];  // Data per burst
  // End of native variables

  private NativeHandler nh;
  private int time = 0;
  private int maxTime = 2000;  // ms
  private double dt = 1000.0 / sampleRate;  // ms
  private GPanel p  = new GPanel(0, maxTime, min, max);
  private volatile boolean isRunning = false;
  private double offset0 =  4; // y-offset of AI0
  private double offset1 = -4; // y-offset of AI1
  private String title = "NI ADC - x: " + (maxTime / 10) + 
            " ms/div; y: " + (max - min) / 10 + " V/div";

  public JawEx7()
  {
    p.title("Initializing...");
    drawCoordinateSystem();
    p.addExitListener(this);
    nh = new NativeHandler(
       "s:\\myjni\\nidaqc\\release\\nidaqc.dll", 
       this);

    if (isError(nh.invoke(0)))
      return;
    if (isError(nh.invoke(1)))
      return;
    if (isError(nh.invoke(2)))
      return;
    if (isError(nh.invoke(3)))
      return;
    if (isError(nh.invoke(4)))
      return;

    p.title(title);
    int pointsRead;  // Number of samples in one burst
    int oldTime = 0;
    double oldEndData0 = 0;
    double oldEndData1 = 0;
    isRunning = true;
    while (isRunning)
    {
      while (time < maxTime)
      {
        // Get data block by block
        pointsRead = nh.invoke(5);
        if (pointsRead == 0)
        {
          p.title("Error while reclaiming data");
          isRunning = false;
          break;
        }

        // Draw signal AI0
        p.color(Color.red);
        p.move(oldTime, oldEndData0);
        for (int i = 0; i < pointsRead; i++)
          p.draw(time + i * dt, data[2*i] + offset0);
        oldEndData0 = data[2*(pointsRead-1 )] + offset0;

        // Draw signal AI1
        p.color(Color.green);
        p.move(oldTime, oldEndData1);
        for (int i = 0; i < pointsRead; i++)
          p.draw(time + i * dt, data[2*i+1] + offset1);
        oldEndData1 = data[2*(pointsRead-1) + 1] + offset1;

        time += pointsRead * dt;
        oldTime = time;
      }
      if (!isRunning)
        break;
      p.clear();
      drawCoordinateSystem();
      oldTime = 0;
      oldEndData0 = 0;
      oldEndData1 = 0;
      time = 0;
    }
  }

  public void notifyExit()
  {
    if (isRunning)
    {
      isRunning = false;
      p.title(title + " - stopped");
    }
    else
    {
      nh.destroy();
      System.exit(0);
    }
  }

  private void drawCoordinateSystem()
  {
    p.color(Color.black);
    double xspan = maxTime / 10;
    for (int i = 0; i <= 10; i++)
      p.line(i * xspan, -10, i * xspan, 10);
    double yspan = (max - min) / 10;
    for (int i = 0; i <= 10; i++)
      p.line(0, min + i * yspan, maxTime, min + i * yspan);
  }

  private boolean isError(int rc)
  {
    if (rc == -1)
    {
      p.title(errorDescription);
      nh.destroy();
      return true;
    }
    return false;
  }

  public static void main(String args[])
  {
    new JawEx7();
  }
}
