// JawEx8.java

import ch.aplu.jaw.*;
import ch.aplu.util.*;

public class JawEx8 implements ExitListener
{
  // Native variables
  private String device = "usb-ad";
  private int period = 20;  // ms
  private int maxTime = 10000; // ms
  private double range = 5; // +- 5V
  // End of native variables

  private NativeHandler nh;
  private GPanel p = new GPanel(0, maxTime, -range, range);

  public JawEx8()
  {
    p.addExitListener(this);
    nh = new NativeHandler(
      "s:\\myjni\\bmc\\release\\bmc.dll", this);
    double[] values = new double[1];  // Array of 1 double
    nh.createBuf(values);  // JNIBuffer with default bufSize
    nh.startThread();  // Start high priority thread

    int time = 0;
    while (true)
    {
      int nbPending = nh.countBuf();
      // Show number of pending data on heavily loaded systems
      if (time > 0 && time % 200 == 0)
        p.title("Buffer Count: " + nbPending);

      // Read exactly one element from buffer
      if (nh.readBuf(1) == 1)
      {
        if (time == 0)
        {
          p.clear();
          drawCoordinateSystem();
          p.move(time, values[0]);
        }
        else
          p.draw(time, values[0]);
        time += period;
        if (time > maxTime)
          return;
//          time = 0;
      }
       // Sleep 1 ms if few pending data
       // in order to decrease CPU load substantially
      if (nbPending < 2)
        Console.delay(1); 
    }
  }

  private void drawCoordinateSystem()
  {
    double xspan = maxTime / 10;
    for (int i = 0; i <= 10; i++)
      p.line(i * xspan, -10, i * xspan, 10);
    double yspan = 2*range / 10;
    for (int i = 0; i <= 10; i++)
      p.line(0, -range + i * yspan, 
             maxTime, -range + i * yspan);
  }
  
  public void notifyExit()
  {
    nh.destroy();
    System.exit(0);
  }

  public static void main(String args[])
  {
    new JawEx8();
  }
}
