// BMCHandler.cpp
// Must link with libad4.lib, unless using simulation mode

#include "BMCHandler.h"
#include <math.h>

#define simulation

// ----------------- Selector --------------------------------
WindowHandler * 
  selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
  return new AnalogIn(env, obj);
}

// ----------------- class AnalogIn --------------------------
AnalogIn::AnalogIn(JNIEnv * env, jobject obj)
  : WindowHandler(env, obj)
{
  const char * driver = 
    JNIString(_exposedObj, "device").getMBCS();
  period = JNIVar<int>(_exposedObj, "period").get();
  maxTime = JNIVar<int>(_exposedObj, "maxTime").get();
  range = JNIVar<double>(_exposedObj, "range").get();
#ifndef simulation
  adHandle = ad_open(driver);
#endif
  delete [] driver;
}

 AnalogIn::~AnalogIn()
 {
#ifndef simulation
   if (adHandle >= 0)
     ad_close(adHandle);
#endif
 }

 bool AnalogIn::wantThreadMsg(UINT uMsg)
{
  return (uMsg == WM_THREADSTART || 
                  WM_TIMER ||
                  WM_THREADSTOP);
 }

void AnalogIn::evThreadMsg(HWND /*hwnd*/, 
                           UINT message, 
                           WPARAM /* wParam */, 
                           LPARAM /* lParam */)
// Runs in JNIThread and not in the normal application thread
{
  #define ID_TIMER 1
  double value;
  static double time = 0;

  switch (message)
  {
    case WM_THREADSTART:
      SetTimer(0, ID_TIMER, period, (TIMERPROC)NULL);
      break;

    case WM_TIMER:
#ifdef simulation
      value = range*exp(-0.0004*time)*sin(0.005*time);
		  time += period;
		  if (time > maxTime)
        time = 0;
#else
      int32_t channel = 1;
      uint32_t data;
      ad_discrete_in(adHandle, 
                     AD_CHA_TYPE_ANALOG_IN | channel, 
                     0, &data);
	  	// Interprete data as 32-bit 2's complement 
      // and convert to volts 
      value = range * (int)data / (double)0x80000000;
#endif
	 	  // Copy to JNIBuffer
      writeDouble(value);
      break;
    
    case WM_THREADSTOP:
      KillTimer(0, ID_TIMER);
      break;
  }
}
