// MouseEventHandler.cpp

#include "MouseEventHandler.h"

// ----------------- Selector --------------------------------
WindowHandler * 
  selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
  return new MouseEventHandler(env, obj);
}

// ----------------- class MouseEventHandler  ----------------
// Constructor, initialize base class
MouseEventHandler::MouseEventHandler(JNIEnv * env, jobject obj)
  : WindowHandler(env, obj)
{}

// Select which windows message will call evMsg()
bool MouseEventHandler::wantMsg(UINT uMsg)
{
  return (uMsg == WM_CREATE ||
                  WM_LBUTTONDOWN ||
                  WM_LBUTTONUP ||
                  WM_LBUTTONDBLCLK ||
                  WM_MOVE ||
   	              WM_PAINT ||
				          WM_DESTROY);
}

// Event procedure
// Return false, if message is handled by this procedure, 
// true to invoke DefWindowProc()
bool MouseEventHandler::evMsg(HWND hWnd, UINT message, 
                        WPARAM wParam , LPARAM lParam)
{
  static HBITMAP hBm = NULL;
  static int xPos;
  static int yPos;
  HDC hDC;
  HDC hDCMem;
  BITMAP bm;
  PAINTSTRUCT ps;

  switch (message)
  {
    case WM_CREATE:
      hBm = LoadBitmap(getInstance(), TEXT("ROSE"));
      return false;

    case WM_LBUTTONDOWN:
      xPos = LOWORD(lParam);  // Relative coordinates
      yPos = HIWORD(lParam);
	    sendMouseMessage(LPRESS, xPos, yPos);
	    return false;

   case WM_LBUTTONUP:
	   sendMouseMessage(LRELEASE, LOWORD(lParam), HIWORD(lParam));
     return false;

   case WM_LBUTTONDBLCLK:
	   sendMouseMessage(LDCLICK, LOWORD(lParam), HIWORD(lParam));
     return false;

    case WM_MOUSEMOVE:
      if (wParam == MK_LBUTTON)
      {
        int xNew = LOWORD(lParam);  // Relative coordinates
        int yNew = HIWORD(lParam);
        int xDelta = xNew - xPos;   // Displacement
        int yDelta = yNew - yPos;
        _xPos += xDelta;            // Absolute coordinates
        _yPos += yDelta;            // of upper left corner
        SetWindowPos(hWnd, HWND_TOPMOST, _xPos, _yPos, 0, 0, 
                     SWP_NOSIZE | SWP_SHOWWINDOW);
      }
      return false;

    case WM_PAINT:
      hDC = BeginPaint(hWnd, &ps);
      hDCMem = CreateCompatibleDC(hDC);
      SelectObject(hDCMem, hBm);
      GetObject(hBm, sizeof(bm), &bm);
      BitBlt(hDC, 0, 0, bm.bmWidth, bm.bmHeight, 
             hDCMem, 0, 0, SRCCOPY);
      DeleteDC(hDCMem);
      EndPaint(hWnd, &ps);
	    return false;

    case WM_DESTROY:
      DeleteObject(hBm) ;
      break;
  }
  return true;
}

void MouseEventHandler::sendMouseMessage(int message, int x, int y)
{
  JNIMethod<void> mouseEvent(_exposedObj, "mouseEvent", "III");
  jvalue par[3];
  par[0].i = message;
  par[1].i = x;
  par[2].i = y;
  mouseEvent.call(par);
}

