// NIDAQHandler.cpp
// National Instruments NI-DAQmx
// Prototype acquire1Scan.c from distribution of NI-DAQmx

#include "NIDAQHandler.h"

#define DAQmxErrChk(functionCall) \
  { if (DAQmxFailed(error = (functionCall))) { goto Error; } }

// ----------------- Selector --------------------------------
WindowHandler * 
  selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
  return new NIDAQAcquire1Scan(env, obj);
}

// ----------------- class NIDAQAcquire1Scan -----------------
NIDAQAcquire1Scan::NIDAQAcquire1Scan(JNIEnv * env, jobject obj)
   : WindowHandler(env, obj), 
     samplesPerChan(1), 
     timeout(10.0),
     pointsToRead(1)
{
  period = JNIVar<int>(_exposedObj, "period").get();
  chan = JNIString(_exposedObj, "channel").getUTF8();
  min = JNIVar<double>(_exposedObj, "min").get();
  max = JNIVar<double>(_exposedObj, "max").get();
}

NIDAQAcquire1Scan::~NIDAQAcquire1Scan()
{
  delete [] chan;
  if(taskHandle != 0)
  {
    DAQmxBaseStopTask(taskHandle);
    DAQmxBaseClearTask(taskHandle);
  }
}

int NIDAQAcquire1Scan::invoke(int val)
// Returns 0, if successful, -1, if error
{
  switch (val)
  {
    case 0:  // Create task
      DAQmxErrChk(DAQmxBaseCreateTask("", &taskHandle));
      break;

    case 1:  // Open channel
      DAQmxErrChk(
        DAQmxBaseCreateAIVoltageChan(taskHandle,
                                     chan, "",
                                     DAQmx_Val_RSE,
                                     min,
                                     max,
                                     DAQmx_Val_Volts,
                                     NULL));
      break;

      case 2:  // Start task
        DAQmxErrChk(DAQmxBaseStartTask(taskHandle));
        break;

     case 3:  // Get data
       DAQmxErrChk(
         DAQmxBaseReadAnalogF64(taskHandle,
                                pointsToRead,
                                timeout,
                                DAQmx_Val_GroupByChannel,
                                &data,
                                samplesPerChan,
                                &pointsRead,
                                NULL));
       JNIVar<double>(_exposedObj, "value").set(data);
       break;

      case 4:  // Sleep
         Sleep(period);
         break;
   }
   return 0;

Error:
  if (DAQmxFailed(error))
    DAQmxBaseGetExtendedErrorInfo(errBuff, 2048);
  if (taskHandle!=0) 
  {
    DAQmxBaseStopTask(taskHandle);
    DAQmxBaseClearTask(taskHandle);
  }
  if (DAQmxFailed(error))
  {
   JNIString(_exposedObj, "errorDescription").setMBCS(errBuff);
   return -1;
	}
	return 0;
}
