// NIDAQHandler.cpp
// National Instruments NI-DAQmx
// Prototype contAcquireNScan.c from distribution of NI-DAQmx

#include "NIDAQHandler.h"

#define DAQmxErrChk(functionCall) \
  { if (DAQmxFailed(error = (functionCall))) { goto Error; } }

// ----------------- Selector --------------------------------
WindowHandler * 
  selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
  return new NIDAQContAcquireNScan(env, obj);
}

// ----------------- class NIDAQContAcquireNScan -------------
NIDAQContAcquireNScan::
  NIDAQContAcquireNScan(JNIEnv * env, jobject obj)
   : WindowHandler(env, obj), 
     samplesPerChan(1), 
     timeout(10.0),
     pointsToRead(1)
{
  chan = JNIString(_exposedObj, "channel").getUTF8();
  min = JNIVar<double>(_exposedObj, "min").get();
  max = JNIVar<double>(_exposedObj, "max").get();
  samplesPerChan = 
    JNIVar<__int64>(_exposedObj, "samplesPerChannel").get();
  sampleRate = 
    JNIVar<double>(_exposedObj, "sampleRate").get();

  // Timing parameters
  strcpy(clockSource, "OnboardClock");

  // Data read parameters
  pointsToRead = (int32)samplesPerChan;
}

NIDAQContAcquireNScan::~NIDAQContAcquireNScan()
{
  delete [] chan;
  if(taskHandle != 0)
  {
    DAQmxBaseStopTask(taskHandle);
    DAQmxBaseClearTask(taskHandle);
  }
}

int NIDAQContAcquireNScan::invoke(int val)
// Returns 0, if successful, -1, if error
{
  switch (val)
  {
    case 0:  // Create task
      DAQmxErrChk(DAQmxBaseCreateTask("", &taskHandle));
      break;

    case 1:  // Open channel
      DAQmxErrChk(
        DAQmxBaseCreateAIVoltageChan(taskHandle,
                                    chan, "",
                                    DAQmx_Val_RSE,
                                    min, max,
                                    DAQmx_Val_Volts,
                                    NULL));
      break;

    case 2:  // Configure internal clock
      DAQmxErrChk(
        DAQmxBaseCfgSampClkTiming(taskHandle,
                                  clockSource,
                                  sampleRate,
                                  DAQmx_Val_Rising,
                                  DAQmx_Val_ContSamps,
                                  samplesPerChan));
      break;
   
    case 3:  // Allocate buffer
      DAQmxErrChk(
        DAQmxBaseCfgInputBuffer(taskHandle, 200000));
      break;

    case 4:  // Start task
      DAQmxErrChk(DAQmxBaseStartTask(taskHandle));
      break;
   
    case 5:  // Acquire data
      JNIArray<double> jary(_exposedObj, "data");
      double * data = jary.get();

      DAQmxErrChk(
        DAQmxBaseReadAnalogF64(taskHandle,
                               pointsToRead,
                               timeout,
                               DAQmx_Val_GroupByScanNumber,
                               data,
                               2*(int)samplesPerChan,
                               &pointsRead,
                               NULL));
      return pointsRead;
   }
   return 0;


Error:
  if (DAQmxFailed(error))
    DAQmxBaseGetExtendedErrorInfo(errBuff, 2048);
  if (taskHandle!=0) 
  {
    DAQmxBaseStopTask(taskHandle);
    DAQmxBaseClearTask(taskHandle);
  }
  if (DAQmxFailed(error))
  {
    JNIString(_exposedObj, "errorDescription").
      setMBCS(errBuff);
    return -1;
	}
	return 0;
}
