// JNIString.cpp

#include "JNIString.h"
#include "WindowThread.h"

// disable strcpy warning with Visual Studio
#pragma warning(disable : 4996) 

extern WindowThread threadInstance; // Thread instance from WindowsThread

JNIString::JNIString(jobject jObject, const char * name)
{
  if (jObject == 0)
  {
     SHOW_ERROR("Fatal error when creating JNIString", "No object exposed");
  return;
  }

  _env = threadInstance.getEnv();
  _jObject = jObject;
  _pName = new char[strlen(name) + 1];
  strcpy(_pName, name);
  strcpy(_type, "Ljava/lang/String;");   // Don't forget the ;
  _id = getFieldID();
}

JNIString::~JNIString()
{
  if (_pName)
    delete [] _pName;
}

jfieldID JNIString::getFieldID()
{
   char msg[128];
//   cout << "looking for variable " << _pName << " with type: " << _type << endl;
   _id = _env->GetFieldID(_env->GetObjectClass(_jObject), _pName, _type);

   if (!_id)
   {
     strcpy(msg, "Variable ");
     strcat(msg, _pName);
     strcat(msg, " not found.");
  SHOW_ERROR("Fatal error in JNIString::getFieldID()", msg)
     return 0;
   }
   return _id;
}


const wchar_t * JNIString::getUnicode()
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::getUnicode()", "Illegal type");
      return NULL;
   }
   jboolean isCopy;
   jstring jstr = (jstring)_env->GetObjectField(_jObject, _id);
   const jchar * str = _env->GetStringChars(jstr, &isCopy);
   int len = int(wcslen((wchar_t *)str));
   wchar_t * unicodestr = new wchar_t[len+1];
   wcscpy(unicodestr, (wchar_t *)str);
   if (isCopy == JNI_TRUE)
      _env->ReleaseStringChars(jstr, str);
   return unicodestr;  // Must be deleted by user
}

const wchar_t * JNIString::getUnicode(JNIEnv * env, jstring jstr)
{
   jboolean isCopy;
   const jchar * str = env->GetStringChars(jstr, &isCopy);
   int len = int(wcslen((wchar_t *)str));
   wchar_t * unicodestr = new wchar_t[len+1];
   wcscpy(unicodestr, (wchar_t *)str);
   if (isCopy == JNI_TRUE)
      env->ReleaseStringChars(jstr, str);
   return unicodestr;  // Must be deleted by user
}

const char * JNIString::getMBCS()
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::getMBCS()", "Illegal type");
      return NULL;
   }
   jboolean isCopy;
   jstring jstr = (jstring)_env->GetObjectField(_jObject, _id);
   const jchar * str = _env->GetStringChars(jstr, &isCopy);
   int len = int(wcslen((wchar_t*)str));
   char * asciistr = new char[len + 1];
   WideCharToMultiByte(CP_ACP, 0, (wchar_t *)str, -1, asciistr, len+1, NULL, NULL);

   if (isCopy == JNI_TRUE)
      _env->ReleaseStringChars(jstr, str);
   return asciistr;  // Must be deleted by user
}

const char * JNIString::getMBCS(JNIEnv * env, jstring jstr)
{
   jboolean isCopy;
   const jchar * str = env->GetStringChars(jstr, &isCopy);
   int len = int(wcslen((wchar_t*)str));
   char * asciistr = new char[len + 1];
   WideCharToMultiByte(CP_ACP, 0, (wchar_t *)str, -1, asciistr, len+1, NULL, NULL);

   if (isCopy == JNI_TRUE)
      env->ReleaseStringChars(jstr, str);
   return asciistr;  // Must be deleted by user
}

const char * JNIString::getUTF8()
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::getUTF8()", "Illegal type");
      return NULL;
   }
   jboolean isCopy;
   jstring jstr = (jstring)_env->GetObjectField(_jObject, _id);
   const char * str = _env->GetStringUTFChars(jstr, &isCopy);
   char * pStr = new char[strlen(str)+1];
   strcpy(pStr, str);
   if (isCopy == JNI_TRUE)
      _env->ReleaseStringUTFChars(jstr, str);
   return pStr;  // Must be deleted by user
}

const char * JNIString::getUTF8(JNIEnv * env, jstring jstr)
{
   jboolean isCopy;
   const char * str = env->GetStringUTFChars(jstr, &isCopy);
   char * pStr = new char[strlen(str)+1];
   strcpy(pStr, str);
   if (isCopy == JNI_TRUE)
      env->ReleaseStringUTFChars(jstr, str);
   return pStr;  // Must be deleted by user
}

void JNIString::setUnicode(const wchar_t * value)
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::setUnicode()", "Illegal type");
      return;
   }
   int len = int(wcslen(value));
   jstring jstr = _env->NewString((jchar *)value, (jsize)len);
   _env->SetObjectField(_jObject, _id, jstr);
}

void JNIString::setMBCS(const char * value)
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::setMBCS()", "Illegal type");
      return;
   }

   int len = MultiByteToWideChar(CP_ACP, 0, value, -1, NULL, 0); // len of buf including terminator
   wchar_t * unicodestr = new wchar_t[len];
   MultiByteToWideChar(CP_ACP, 0, value, -1, unicodestr, len);
   jstring jstr = _env->NewString((jchar *)unicodestr, (jsize)(len-1));
   _env->SetObjectField(_jObject, _id, jstr);
   delete [] unicodestr;
}

void JNIString::setUTF8(const char * value)
{
   if (_type[0] != 'L')
   {
      SHOW_ERROR("Fatal error in JNIString::setUTF8()", "Illegal type");
      return;
   }
   jstring jstr = _env->NewStringUTF(value);
   _env->SetObjectField(_jObject, _id, jstr);
}

