// WindowHandler.cpp
// For jaw_bc.lib compile normally

// For jaw_vc.lib compile using option
//   Multithreaded (/MT)
//   Use wchar_t as type
//   Use runtime information

// For jaw_mfc.lib compile with options
//   Multithreaded (/MT)
//   Use wchar_t as type
//   Use runtime information
// dllmodul_jaw.cpp must be present (slightly modified from vc-distribution
// Symbol JAW_MFC must be defined

// For Visual Studio 2008 Express edition, Release
// Projekt als static library, ohne precompiled header
// Konfigurationseigenschaften | Allgemein | Zeichensatz | Multi-Byte-Zeichensatz verwenden
// Compiler: Zusaetzliches Include-Verzeichnis: ..\myjni\include
// Linker: Ausgabedatei ..\jaw.lib


// disable strcpy warning with Visual Studio
#pragma warning(disable : 4996) 

#define _WIN32_IE 0x0500  // To use correct version of NOTIFYICONDATA

#ifdef JAW_MFC
#define WINVER 0x0501
// #include "dllmodul_jaw.cpp"  // Modified source from VC distribution, see comments in dllmodul_jaw.cpp
#endif


#define WIN32_LEAN_AND_MEAN      // Exclude rarely used stuff from Windows headers

#include <windows.h>
#include <shellapi.h>
#include <shlwapi.h>
#include <stdlib.h>
#include "WindowHandler.h"
#include "WindowThread.h"
#include "WindowList.h"
#include "JNIBuffer.cpp"
#include "JNIString.h"

#include <iostream>
using namespace std;

extern WindowThread threadInstance; // Thread instance from NativeWindowsThread

// Static definitions
HINSTANCE WindowHandler::_hInstance = 0;
WindowThread * WindowHandler::_pThread;
PSLWA WindowHandler::pSetLayeredWindowAttributes;
bool WindowHandler::isKernelInitialized = false;

#ifndef JAW_MFC
// Entry point of DLL. Use to get the HINSTANCE
// NEVER USE DllEntryPoint() -> not called when compiling with VC-library!!!!
BOOL WINAPI DllMain(HINSTANCE hinstDll,
                           DWORD fdwReason,
                           LPVOID /*plvReserved */)
{
   switch(fdwReason)
   {
     case DLL_PROCESS_ATTACH:
//        cout << "DLL_PROCESS_ATTACH" << endl;
        WindowHandler::setInstance(hinstDll);
        break;

     case DLL_PROCESS_DETACH:
//        cout << "DLL_PROCESS_DETACH" << endl;
        break;
   }
   return true;
}
#endif

// ------------------------------ Ctor -----------------------------------
WindowHandler::WindowHandler(JNIEnv * env, jobject handlerObj)
{
   jfieldID id;
   _hwnd = 0;
   hIcon = 0;
   isTrayIcon = false;
   isInitDone = false;
   isTransparent = false;
   
   _pBoolBuf = 0;
   _pCharBuf = 0;
   _pByteBuf = 0;
   _pShortBuf = 0;
   _pIntBuf = 0;
   _pLongBuf = 0;
   _pFloatBuf = 0;
   _pDoubleBuf = 0;
   _exposedObj = 0;
   _pIconEventBuf = 0;
   
   _pThread = &threadInstance;
   _pThread->setEnv(env);   // Current env is stored here, for all window the same

   // Make NativeHandler reference global, so each JNI method can use it
   _handlerObj = env->NewGlobalRef(handlerObj); 

   // Set the instance variable 'handler' in NativeWindow.java to the this-pointer
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "ref", "J");
   env->SetLongField(_handlerObj, id, (jlong)this);

   // Get parameters from Java handler's instance variables
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "exposedObj", "Ljava/lang/Object;");
   jobject exposedObj = (jobject)env->GetObjectField(handlerObj, id);
   expose(exposedObj);

   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "title", "Ljava/lang/String;");
   jstring jstr = (jstring)env->GetObjectField(_handlerObj, id);
   jboolean isCopy;
   const jchar * str = env->GetStringChars(jstr, &isCopy);
   int len = int(wcslen((wchar_t *)str));
   wchar_t * unicodestr = new wchar_t[len+1];
   wcscpy(unicodestr, (wchar_t *)str);
   _title = new char[len+1];
   if (len > 0)
     WideCharToMultiByte(CP_ACP, 0, unicodestr, -1, _title, len+1, NULL, NULL);
   else
      lstrcpy(_title, "");
   if (isCopy == JNI_TRUE)
      env->ReleaseStringChars(jstr, str);

   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "windowStyle", "J");
   _winStyle = (DWORD)(env->GetLongField(_handlerObj, id));
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "xPos", "I");
   _xPos = (int)(env->GetIntField(_handlerObj, id));
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "yPos", "I");
   _yPos = (int)(env->GetIntField(_handlerObj, id));
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "width", "I");
   _winWidth = (int)(env->GetIntField(_handlerObj, id));
   id = env->GetFieldID(env->GetObjectClass(_handlerObj), "height", "I");
   _winHeight = (int)(env->GetIntField(_handlerObj, id));
}

// ------------------------------ Dtor -----------------------------------
WindowHandler::~WindowHandler()
{
//   cout << "calling Dtor()..." << endl;
   delete pJNIThread;
   delete [] _title;      
}

void WindowHandler::destroyBuf()
{
   if (_pBoolBuf)   { delete _pBoolBuf;   _pBoolBuf = 0;  } else
   if (_pCharBuf)   { delete _pCharBuf;   _pCharBuf = 0;  } else
   if (_pByteBuf)   { delete _pByteBuf;   _pByteBuf = 0;  } else
   if (_pShortBuf)  { delete _pShortBuf;  _pShortBuf = 0; } else
   if (_pIntBuf)    { delete _pIntBuf;    _pIntBuf = 0;   } else
   if (_pLongBuf)   { delete _pLongBuf;   _pLongBuf = 0;  } else
   if (_pFloatBuf)  { delete _pFloatBuf;  _pFloatBuf = 0; } else
   if (_pDoubleBuf) { delete _pDoubleBuf; _pDoubleBuf = 0;}
}

void WindowHandler::destroyIconEventBuf()
{
   if (_pIconEventBuf)   
   {
     delete _pIconEventBuf;   
     _pIconEventBuf = 0; 
   }
}

void WindowHandler::expose(jobject obj)
{
//   cout << "exposing " << int(obj) << endl;
   JNIEnv * env = threadInstance.getEnv();
   if (obj != 0)
      _exposedObj = env->NewGlobalRef(obj);
}

// Return pointer to current instance
// Return 0, if hwnd is not found
WindowHandler * WindowHandler::getThis(HWND hwnd)
{
//   cout << "Calling getThis() with hwnd: " << (int)hwnd << endl;
   WindowHandler * wh = threadInstance.getWindowList()->getHandler(hwnd);
   if (wh == 0)
      SHOW_ERROR("Error in WindowHandler::getThis(hwnd)",
                 "Window with given hwnd not found");
   return wh;
}

jobject WindowHandler::getObject(HWND hwnd)
{
   jobject obj = threadInstance.getWindowList()->getObject(hwnd);
   if (obj == 0)
      SHOW_ERROR("Error in WindowHandler::getObject(hwnd)",
                 "Window with given hwnd not found");
   return obj;
}

jobject WindowHandler::getAppObject(HWND hwnd)
{
   return getThis(hwnd)->_exposedObj;
}

void WindowHandler::enable(JNIEnv * env)
{
  threadInstance.create(env); // Create the thread if not already done by another handler

   // Wait unti the thread is up, then we will be called back for execution of doEnable()
   while (!PostThreadMessage(threadInstance.getThreadId(), WM_NativeWindow, enableCode, (LPARAM)this))
      Sleep(0);
   while (!isInitDone)  // Wait until native window is fully initialized
      Sleep(0);
   pJNIThread = new JNIThread(this, THREAD_PRIORITY_TIME_CRITICAL);
}

void WindowHandler::showWindow(bool isVisible, int percent, int red, int green, int blue, bool wait)
{
  if (isVisible)
  { 
//     cout << "r: " << red << " g: " << green << " b: "  << blue << endl;
     if (isTransparent && percent >= 0 && percent <= 255 )
     {
        BYTE factor = (percent * 255) / 100;
        BOOL rc = 
           pSetLayeredWindowAttributes(_hwnd, RGB(red, green, blue), 
                                       factor,
                                       LWA_COLORKEY | LWA_ALPHA);
        if (rc == 0)
        {
           SHOW_ERROR("Fatal error in WindowHandler::show()", 
                      "SetLayeredWindowAttributes() failed");
           return;
        }
      }
      SetWindowPos(_hwnd, HWND_TOP, _xPos, _yPos, _winWidth, _winHeight, SWP_SHOWWINDOW);
      InvalidateRect(_hwnd, 0, true);
      UpdateWindow(_hwnd);
      if (wait)
      {
         int n = 0;
         do
         {
            Sleep(10);
            n++;
         } while (!isImageLoaded && n < 500);  // Wait max 5 s
      }
   }
   else
      ShowWindow(_hwnd, false);
}

void WindowHandler::setWindowPosition(int xPos, int yPos)
{
   _xPos = xPos;
   _yPos = yPos;
/*
   UINT uFlags;
   if (IsWindowVisible(_hwnd))
      uFlags = SWP_SHOWWINDOW;
   else
      uFlags = SWP_SHOWWINDOW;
*/
//   SetWindowPos(_hwnd, HWND_TOP, _xPos, _yPos, _winWidth, _winHeight, uFlags);
   MoveWindow(_hwnd, _xPos, _yPos, _winWidth, _winHeight, true);
}

void WindowHandler::setWindowSize(int width, int height)
{
   _winWidth = width;
   _winHeight = height;
   UINT uFlags;
   if (IsWindowVisible(_hwnd))
      uFlags = SWP_SHOWWINDOW;
   else
      uFlags = SWP_SHOWWINDOW;

   SetWindowPos(_hwnd, HWND_TOP, _xPos, _yPos, _winWidth, _winHeight, uFlags);
}

// Callback function from thread's Window Procedure
void WindowHandler::doEnable()
{
   // className must be different for each window, so we append the refcount
   char className[64] = "ApiNativeWindowClass";
   char count[10];
   _itoa(threadInstance.getHandlerCount(), count, 10);  // itoa is depreciated
   strcat(className, count);

//   cout << "Setting up WNDCLASSEX. className:" << className << endl;

   // Register window class
   WNDCLASSEX wndclass;
   wndclass.cbSize = sizeof(wndclass);
   wndclass.style = CS_HREDRAW |
                    CS_SAVEBITS | 
                    CS_VREDRAW | 
                    CS_DBLCLKS;
   wndclass.lpszClassName = TEXT(className);
   wndclass.lpfnWndProc = WndProc;
   wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
   wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
   wndclass.hIcon = NULL;
   wndclass.hIconSm = NULL;
   wndclass.lpszMenuName = NULL;
   wndclass.cbClsExtra = 0;
   wndclass.cbWndExtra = 0;

   if( !RegisterClassEx(&wndclass) )
   {
      SHOW_ERROR("Fatal error", "Failed to register window class");
      return;
   }

//    cout << "Calling CreateWindow. winStyle: " << winStyle << endl;
   // Create window


   _hwnd = CreateWindow
   ( 
      TEXT(className),
      TEXT(_title),
      _winStyle & ~WS_VISIBLE, // Don't show window here, wait until WindowList is setup
      _xPos, _yPos, _winWidth, _winHeight,
      NULL,
      NULL,
      0,
      (LPVOID)this   // Pass this-pointer to Window Procedure in order to treat WM_CREATE specially
   );

   if (_hwnd == 0)
   {
     SHOW_ERROR("Fatal error", "Failed to create window");
     return;
   }

	 if (!isKernelInitialized && (_winStyle & WS_EX_TRANSPARENT))
	 {
      isKernelInitialized = true;
		  HMODULE hDLL = LoadLibrary ("user32");
  		pSetLayeredWindowAttributes = 
  			  (PSLWA) GetProcAddress(hDLL, "SetLayeredWindowAttributes");
      if (pSetLayeredWindowAttributes == NULL)
      {
        SHOW_ERROR("Fatal error", "Transparent windows not supported\nby the current operation system.\nMust be Windows 2000 or higher.");
        return;
      }
   }

   // Insert into WindowList using the global object reference
   threadInstance.getWindowList()->insert(_hwnd, _handlerObj, this);

   if (_winStyle & WS_EX_TRANSPARENT)
   {
      SetWindowLong(_hwnd, GWL_EXSTYLE, WS_EX_LAYERED);
      isTransparent = true;
   }

   if (_winStyle & WS_VISIBLE)
   {
     ShowWindow(_hwnd,  true) ;
     UpdateWindow(_hwnd) ;
   }

   isInitDone = true;
}

void WindowHandler::disable()
/* Rather weired process for cleanup:
- post notification to ThreadInstance via special val 'disableCode'
- in thread's Window Procedure: call handler's method 'doDisable()'
    - delete handler instance: destructor will be called
- in WindowProcedure some messages will not be dispatched, because
  the handler's item in WindowList is already removed
*/
{
   isDetached = false;
//   cout << "calling disable()..." << endl;
   int count = _pThread->handlerCount;

   // Release our reference here, not in dtor, because here Java environment is still valable
   threadInstance.getEnv()->DeleteGlobalRef(_handlerObj);
   if (_exposedObj != 0)
      threadInstance.getEnv()->DeleteGlobalRef(_exposedObj);

   killJNIThread();
   destroyBuf();
   destroyIconEventBuf();
   deleteTrayIcon();
   SendMessage(_hwnd, WM_CLOSE, 0, 0);
   threadInstance.getWindowList()->remove(_hwnd);
   PostThreadMessage(threadInstance.getThreadId(), WM_NativeWindow, disableCode, (LPARAM)this);
   if (count == 1)  // Must wait until VM is released
   {
      int n = 0;
      while (!isDetached && n++ < 200)  // Wait max 2 s
         Sleep(0);
//      cout << "must wait n = " << n << endl;
   }
//   cout << "disable() finished" << endl;
}

// Window Procedure
LRESULT CALLBACK WindowHandler::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
   // Must treat WM_CREATE specially, because handle to window is not yet inserted in WindowList
   // We got the pointer to the object from lParam, because we stored it in CreateWindow

 //  static int nb = 0; cout << nb++ << ": Message hex " << hex << uMessage << "  sent to window hwnd: " << hWnd << endl;


   if (uMessage == WM_CREATE)
   {
      CREATESTRUCT * pC = (CREATESTRUCT *)lParam;
      WindowHandler * createThis = (WindowHandler *)(pC->lpCreateParams);
      if (createThis->wantMsg(uMessage))
         if (createThis->evMsg(hWnd, uMessage, wParam, lParam))
            return 0;
   }

   if (uMessage == WM_TASKBARICON)
      getThis(hWnd)->iconCallback(int(lParam));

   if (uMessage == WM_TIMER && wParam == ID_TIMER_ICON )
      getThis(hWnd)->iconCallback(WM_TIMER);
   
   jobject jObject = threadInstance.getWindowList()->getObject(hWnd);
   if (jObject)  // object found with given hwnd
   {
//      cout << "Message hex " << hex << uMessage << "  sent to window hwnd: " << hWnd << endl;
      if (!(uMessage == WM_MOUSELEAVE) || IsWindowVisible(hWnd))  // Do not create LEAVE event when window is made hidden
      {
         if (getThis(hWnd)->wantMsg(uMessage))
            if (!getThis(hWnd)->evMsg(hWnd, uMessage, wParam, lParam))
			{
   //            cout << "Message hex " << hex << uMessage << "  not sent to DefWindowProc()" << endl;
               return 0;
			}
      }
   }
   else
   {
//     cout << "Message " << hex << uMessage << "  NOT sent to window hwnd: " << hWnd << endl;
   }
   // cout << "Message hex " << hex << uMessage << "  sent to DefWindowProc()" << endl;
   return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

// Dummy method to be overridden by user
bool WindowHandler::wantMsg(UINT)
{
   return false;  // We do not want any message
}

// Dummy method to be overridden by user
bool WindowHandler::wantThreadMsg(UINT)
{
   return false;  // We do not want any message
}

// Dummy method to be overridden by user
bool WindowHandler::evMsg(HWND, UINT, WPARAM, LPARAM)
{
   return true;  // All messages passed to Windows
}

// Dummy method to be overridden by user
void WindowHandler::evThreadMsg(HWND, UINT, WPARAM, LPARAM)
{
   return;
}

// Method to be overridden by user
int WindowHandler::invoke(int)
{
   return 0;  // No-error flag
}

jfieldID WindowHandler::getFieldID(jobject jObject, const char * name, const char * type)
{
   char msg[128];
   JNIEnv * env = threadInstance.getEnv();
   jfieldID id = env->GetFieldID(env->GetObjectClass(jObject), name, type);
   if (!id)
   {
     strcpy(msg, "Variable ");
     strcat(msg, name);
     strcat(msg, " not found.");
	 SHOW_ERROR("Fatal error in WindowHandler::getFieldID()", msg);
     return 0;
   }
   return id;
}

jobject WindowHandler::getObjectVariable(jobject jObject, const char * name, const char * classname)
{
   char msg[128];
   char signature[128] = "L";
   strcat(signature, classname);
   strcat(signature, ";");
   JNIEnv * env = threadInstance.getEnv();
   jfieldID id = env->GetFieldID(env->GetObjectClass(jObject), name, signature);
   if (!id)
   {
     strcpy(msg, "Variable ");
     strcat(msg, name);
     strcat(msg, " not found.");
	 SHOW_ERROR("Fatal error in WindowHandler::getObjectVariable()", msg);
     return 0;
   }

   return (jobject)(env->GetObjectField(jObject, id));
}

void WindowHandler::createBuf(jbooleanArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pBoolBuf = new JNIBuffer<jbooleanArray>(env, jary, size);
}

void WindowHandler::createBuf(jcharArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pCharBuf = new JNIBuffer<jcharArray>(env, jary, size);
}

void WindowHandler::createBuf(jbyteArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pByteBuf = new JNIBuffer<jbyteArray>(env, jary, size);
}

void WindowHandler::createBuf(jshortArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pShortBuf = new JNIBuffer<jshortArray>(env, jary, size);
}

void WindowHandler::createBuf(jintArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pIntBuf = new JNIBuffer<jintArray>(env, jary, size);
}

void WindowHandler::createBuf(jlongArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pLongBuf = new JNIBuffer<jlongArray>(env, jary, size);
}

void WindowHandler::createBuf(jfloatArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pFloatBuf = new JNIBuffer<jfloatArray>(env, jary, size);
}

void WindowHandler::createBuf(jdoubleArray jary, jint size)
{
   destroyBuf();
   JNIEnv * env = threadInstance.getEnv();
   _pDoubleBuf = new JNIBuffer<jdoubleArray>(env, jary, size);
}

int WindowHandler::readBuf(int count)
{
   if (_pBoolBuf)
     return _pBoolBuf->read(count);
   if (_pCharBuf)
     return _pCharBuf->read(count);
   if (_pBoolBuf)
     return _pByteBuf->read(count);
   if (_pShortBuf)
     return _pShortBuf->read(count);
   if (_pIntBuf)
     return _pIntBuf->read(count);
   if (_pLongBuf)
     return _pLongBuf->read(count);
   if (_pFloatBuf)
     return _pFloatBuf->read(count);
   if (_pDoubleBuf)
     return _pDoubleBuf->read(count);
   return -1;
}


void WindowHandler::writeBool(bool value)
{
   if (_pBoolBuf)
      _pBoolBuf->write(value);
}

void WindowHandler::writeChar(wchar_t value)
{
   if (_pCharBuf)
      _pCharBuf->write(value);
}

void WindowHandler::writeByte(signed char value)
{
   if (_pByteBuf)
      _pByteBuf->write(value);
}

void WindowHandler::writeShort(short value)
{
   if (_pShortBuf)
      _pShortBuf->write(value);
}

void WindowHandler::writeInt(int value)
{
   if (_pIntBuf)
      _pIntBuf->write(value);
}

void WindowHandler::writeLong(__int64 value)
{
   if (_pLongBuf)
      _pLongBuf->write(value);
}

void WindowHandler::writeFloat(float value)
{
   if (_pFloatBuf)
      _pFloatBuf->write(value);
}

void WindowHandler::writeDouble(double value)
{
   if (_pDoubleBuf)
      _pDoubleBuf->write(value);
}

void WindowHandler::flushBuf()
{
   if (_pBoolBuf) _pBoolBuf->flush();   else
   if (_pCharBuf) _pCharBuf->flush();   else
   if (_pByteBuf) _pByteBuf->flush();   else
   if (_pShortBuf) _pShortBuf->flush(); else
   if (_pIntBuf) _pIntBuf->flush();     else
   if (_pLongBuf) _pLongBuf->flush();   else
   if (_pFloatBuf) _pFloatBuf->flush(); else
   if (_pDoubleBuf) _pDoubleBuf->flush();
}

int WindowHandler::countBuf()
{
   if (_pBoolBuf)   return _pBoolBuf->count();
   if (_pCharBuf)   return _pCharBuf->count();
   if (_pByteBuf)   return _pByteBuf->count();
   if (_pShortBuf)  return _pShortBuf->count();
   if (_pIntBuf)    return _pIntBuf->count();
   if (_pLongBuf)   return _pLongBuf->count();
   if (_pFloatBuf)  return _pFloatBuf->count();
   if (_pDoubleBuf) return _pDoubleBuf->count();
   return -1;
}

bool WindowHandler::isOverflow(bool reset)
{
   if (_pBoolBuf)   return _pBoolBuf->isOverflow(reset);
   if (_pCharBuf)   return _pCharBuf->isOverflow(reset);
   if (_pByteBuf)   return _pByteBuf->isOverflow(reset);
   if (_pShortBuf)  return _pShortBuf->isOverflow(reset);
   if (_pIntBuf)    return _pIntBuf->isOverflow(reset);
   if (_pLongBuf)   return _pLongBuf->isOverflow(reset);
   if (_pFloatBuf)  return _pFloatBuf->isOverflow(reset);
   if (_pDoubleBuf) return _pDoubleBuf->isOverflow(reset);
   return false;
}

void WindowHandler::startJNIThread()
{
//   cout << "startThread() calling..." << endl;
   while (!PostThreadMessage(pJNIThread->getThreadId(), WM_NativeWindow, enableCode, (LPARAM)this))
      Sleep(0);
   // Must wait when the thread is created unil it is working...
}

void WindowHandler::stopJNIThread()
{
//   cout << "stopThread() calling..." << endl;
   PostThreadMessage(pJNIThread->getThreadId(), WM_NativeWindow, suspendCode, (LPARAM)this);
}

void WindowHandler::killJNIThread()
{
//   cout << "killThread() calling..." << endl;
   PostThreadMessage(pJNIThread->getThreadId(), WM_NativeWindow, disableCode, (LPARAM)this);
}

void WindowHandler::createTrayIcon()
{
   if (!setIconData())
      return;
   BOOL rc = Shell_NotifyIcon(NIM_ADD, &iconData);
   if (rc)
      isTrayIcon = true;
   else
      SHOW_ERROR("Fatal Error in showTrayIcon()", "Can't notify icon"); 
}

void WindowHandler::modifyTrayIcon()
{
   if (!isTrayIcon)
      return;
   if (!setIconData())
      return;
   BOOL rc = Shell_NotifyIcon(NIM_MODIFY, &iconData);
   if (!rc)
      SHOW_ERROR("Fatal Error in modifyTrayIcon()", "Can't notify icon"); 
}

void WindowHandler::deleteTrayIcon()
{
   if (isTrayIcon)
   {
      DestroyIcon(hIcon);
      Shell_NotifyIcon(NIM_DELETE, &iconData);
      isTrayIcon = false;
   }
}

bool WindowHandler::setIconData()
{
   if (hIcon)
      DestroyIcon(hIcon);

   // Extract icon from file
   const char * iconfile = JNIString(_handlerObj, "iconPath").getMBCS();
   hIcon = ExtractIcon(_hInstance, iconfile, 0);
            
   if(!hIcon)
   {
     char msg[128] = "Can't load ";
     strcat(msg, iconfile);
     SHOW_ERROR("Fatal Error in showIcon()", msg); 
     return false;
   }
   delete [] iconfile;
   
   // Extract tooltip
   const wchar_t * str1 = JNIString(_handlerObj, "tooltip").getUnicode();
   int len = int(wcslen(str1));
   char * tooltip = new char[len + 1];
   WideCharToMultiByte(CP_ACP, 0, str1, -1, tooltip, len+1, NULL, NULL);
   delete [] str1;
   
   // Limit to 63 chars + null
   if (len > 63)
     tooltip[63] = 0;
   
   if (len > 0)
   { 
      lstrcpy(iconData.szTip, TEXT(tooltip));
      iconData.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
   }
   else
      iconData.uFlags = NIF_MESSAGE | NIF_ICON;

    DWORD version = getDllVersion("shell32.dll");
//  cout << "Shell32 version.\nMajor: " << HIWORD(version) << "\nMinor: " << LOWORD(version) << endl;

   if (HIWORD(version) >= 5)
      iconData.cbSize = sizeof(NOTIFYICONDATA);
   else
      iconData.cbSize = NOTIFYICONDATA_V1_SIZE;   // Version 5 define in _WIN32_IE

   iconData.uCallbackMessage = WM_TASKBARICON;
   iconData.uID = 0;
   iconData.hWnd = _hwnd;
   iconData.hIcon = hIcon;
      
   delete [] tooltip;
   return true;
}

void WindowHandler::iconCallback(int msg)
{

   static bool isMonoflopRunning = false;
   static bool isLClickPending = false;
   static bool isRClickPending = false;
   static bool isNextButtonUpIgnored = false;

   switch (msg)
   {
    case WM_LBUTTONDOWN:
      if (iconEventMask & LPRESS)
        reportIconEvent(LPRESS);

      // Start monoflop
      if (!isMonoflopRunning)
      {
        SetTimer(_hwnd, ID_TIMER_ICON, GetDoubleClickTime(), (TIMERPROC)NULL);
        isMonoflopRunning = true;
      }
      return;

    case WM_LBUTTONUP:
      if (iconEventMask & LRELEASE)
        reportIconEvent(LRELEASE);

      if (iconEventMask & LCLICK)  
      {
        if (!(iconEventMask & LDCLICK))  // DClick not active
        {
          if (isMonoflopRunning)
          {
            reportIconEvent(LCLICK);
            isMonoflopRunning = false;
            KillTimer(_hwnd, ID_TIMER_ICON);
          }
        }
        else  // DClick active
        {
          if (!isNextButtonUpIgnored)
            isLClickPending = true;
          else
            isNextButtonUpIgnored = false;
        }
      }
      return;

    case WM_LBUTTONDBLCLK:
      if (iconEventMask & LDCLICK)
      {
        isNextButtonUpIgnored = true;  // DLICK generates a second BUTTONUP, we ignore it
        isLClickPending = false;
        isMonoflopRunning = false;
        KillTimer(_hwnd, ID_TIMER_ICON);
        reportIconEvent(LDCLICK);
      }
      return;

    case WM_RBUTTONDOWN:
      if (iconEventMask & RPRESS)
        reportIconEvent(RPRESS);

      // Start monoflop
      if (!isMonoflopRunning)
      {
        SetTimer(_hwnd, ID_TIMER_ICON, GetDoubleClickTime(), (TIMERPROC)NULL);
        isMonoflopRunning = true;
      }
      return;

    case WM_RBUTTONUP:
      if (iconEventMask & RRELEASE)
        reportIconEvent(RRELEASE);

      if (iconEventMask & RCLICK)  
      {
        if (!(iconEventMask & RDCLICK))  // DClick not active
        {
          if (isMonoflopRunning)
          {
            reportIconEvent(RCLICK);
            isMonoflopRunning = false;
            KillTimer(_hwnd, ID_TIMER_ICON);
          }
        }
        else  // DClick active
        {
          if (!isNextButtonUpIgnored)
            isRClickPending = true;
          else
            isNextButtonUpIgnored = false;
        }
      }
      return;

    case WM_RBUTTONDBLCLK:
      if (iconEventMask & RDCLICK)
      {
        isNextButtonUpIgnored = true;  // DLICK generates a second BUTTONUP, we ignore it
        isRClickPending = false;
        isMonoflopRunning = false;
        KillTimer(_hwnd, ID_TIMER_ICON);
        reportIconEvent(RDCLICK);
      }
      return;
      
    case WM_TIMER:
      KillTimer(_hwnd, ID_TIMER_ICON);
      isMonoflopRunning = false;

      if (isLClickPending)
        reportIconEvent(LCLICK);
      if (isRClickPending)
        reportIconEvent(RCLICK);

      isNextButtonUpIgnored = false;
      isLClickPending = false;
      isRClickPending = false;

      return;

    case WM_MOUSEMOVE:
      if (iconEventMask & MOVE)
        reportIconEvent(MOVE);
      return;
   }
}

void WindowHandler::reportIconEvent(int evt)
{
//   cout << "got iconEvent: " << evt << endl;
   if (_pIconEventBuf)
      _pIconEventBuf->write(evt);
}

void WindowHandler::createIconEventBuf(jintArray jary, jint size)
{
   destroyIconEventBuf();

   // Extract iconEventMask
   JNIEnv * env = threadInstance.getEnv();
   jfieldID id = env->GetFieldID(env->GetObjectClass(_handlerObj), "iconEventMask", "I");
   iconEventMask = (int)(env->GetIntField(_handlerObj, id));

   _pIconEventBuf = new JNIBuffer<jintArray>(env, jary, size);
}

int WindowHandler::readIconEventBuf(jintArray jary, int count)
{
   if (!_pIconEventBuf)
      return -1;
   JNIEnv * env = threadInstance.getEnv();
   return _pIconEventBuf->read(count);
}

void WindowHandler::showBalloonTooltip(int timeout, int flag)
{
   // Extract balloon tooltip title
   const wchar_t * str0 = JNIString(_handlerObj, "balloon_tooltip_title").getUnicode();
   int len0 = int(wcslen(str0));
   char * balloon_tooltip_title = new char[len0 + 1];
   WideCharToMultiByte(CP_ACP, 0, str0, -1, balloon_tooltip_title, len0+1, NULL, NULL);
   delete [] str0;

   // Limit to 63 chars + null
   if (len0 > 63)
     balloon_tooltip_title[63] = 0;

   // Extract balloon tooltip
   const wchar_t * str1 = JNIString(_handlerObj, "balloon_tooltip").getUnicode();
   int len1 = int(wcslen(str1));
   char * balloon_tooltip = new char[len1 + 1];
   WideCharToMultiByte(CP_ACP, 0, str1, -1, balloon_tooltip, len1+1, NULL, NULL);
   delete [] str1;
            
   // Limit to 254 chars + null
   if (len1 > 254)
     balloon_tooltip[254] = 0;

   switch (flag)
   {
      case 0: 
      default:
         iconData.dwInfoFlags = NIIF_NONE;
         break;
           
      case 1: 
         iconData.dwInfoFlags = NIIF_INFO;
         break;

      case 2: 
         iconData.dwInfoFlags = NIIF_WARNING;
         break;

      case 3: 
         iconData.dwInfoFlags = NIIF_ERROR;
         break;         
   }
           
   lstrcpy(iconData.szInfoTitle, TEXT(balloon_tooltip_title));
   lstrcpy(iconData.szInfo, TEXT(balloon_tooltip));
   iconData.uFlags = NIF_INFO;
   iconData.uTimeout = timeout; // in milliseconds

   Shell_NotifyIcon(NIM_MODIFY, &iconData);

   delete [] balloon_tooltip_title;
   delete [] balloon_tooltip;
}

// Code from MSDN
#define PACKVERSION(major,minor) MAKELONG(minor,major)
DWORD WindowHandler::getDllVersion(LPCTSTR lpszDllName)
{
    HINSTANCE hinstDll;
    DWORD dwVersion = 0;

    /* For security purposes, LoadLibrary should be provided with a 
       fully-qualified path to the DLL. The lpszDllName variable should be
       tested to ensure that it is a fully qualified path before it is used. */
    hinstDll = LoadLibrary(lpszDllName);
	
    if(hinstDll)
    {
        DLLGETVERSIONPROC pDllGetVersion;
        pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress(hinstDll, 
                          "DllGetVersion");

        /* Because some DLLs might not implement this function, you
        must test for it explicitly. Depending on the particular 
        DLL, the lack of a DllGetVersion function can be a useful
        indicator of the version. */

        if(pDllGetVersion)
        {
            DLLVERSIONINFO dvi;
            HRESULT hr;

            ZeroMemory(&dvi, sizeof(dvi));
            dvi.cbSize = sizeof(dvi);

            hr = (*pDllGetVersion)(&dvi);

            if(SUCCEEDED(hr))
            {
               dwVersion = PACKVERSION(dvi.dwMajorVersion, dvi.dwMinorVersion);
            }
        }

        FreeLibrary(hinstDll);
    }
    return dwVersion;
}

void WindowHandler::setDetached(bool detached)
{
  isDetached = detached;
}

   