// WindowThread.cpp
// Create an instance, can be used like a 'this'-pointer
// This instance will run out of scope, when the DLL is onloaded,
// so the dtor will be called. This happens when the Java process
// is terminated by System.exit(0)

#include "WindowThread.h"
#include "WindowHandler.h"
#include "WindowList.h"
#include <iostream>

using namespace std;

#ifndef SHOW_ERROR
#define SHOW_ERROR(title, msg) \
   MessageBox(HWND_DESKTOP, msg, title, MB_ICONEXCLAMATION | MB_OK);
#endif

WindowThread threadInstance;

WindowThread::WindowThread()
{
   threadId = 0;
   jniEnv = 0;
   jniThreadEnv = 0;
   pVm = 0;
   handlerCount = 0;
   pWindowList = new WindowList;
}

WindowThread::~WindowThread()
{
//  cout << "Calling ~WindowThread()..." << endl;
  delete pWindowList;
}

void WindowThread::create(JNIEnv * env)
{
   if( !threadId )  // test if thread is already living
   {
//      cout << "Calling CreateThread..." << endl;
//      DWORD tId = GetCurrentThreadId();
//      cout << "ThreadId-create: " << tId << endl;

      // Get VM and initialize pVm
      env->GetJavaVM(&pVm);

      // Start "native" thread
      hThread = CreateThread
      (
         NULL,       // Pointer to thread security attribute
         0,          // Stack size, 0: same as calling process stack
         ThreadProc, // Thread event procedure
         this,       // Value passed to thread's event procedure
         0,          // Creation flag
         &threadId   // Thread identifier
      );
   } 

  
// Does not improve precision of Timer event  
//   SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL);

}
 

// Window Procedure of native thread
DWORD WINAPI WindowThread::ThreadProc(LPVOID lpParameter)
{
//   cout << "ThreadProc. Param: " << (int)lpParameter << endl;
//  DWORD tId = GetCurrentThreadId();
//  cout << "ThreadId-proc: " << tId << endl;

   // Extract this pointer from lpParameter
   WindowThread * pThis = (WindowThread *)lpParameter;

   // Attach the thread to the VM
   pThis->pVm->AttachCurrentThread((void**)&pThis->jniThreadEnv, NULL);

   MSG msg;
   while (GetMessage(&msg, NULL, 0, 0))
   {
      
      if (msg.message == WM_NativeWindow)
      {
         // Extract handler
         WindowHandler * ref = (WindowHandler * )msg.lParam;

         switch (msg.wParam)
         {
           case WindowHandler::enableCode:
             pThis->handlerCount++;
             ref->doEnable();
             break;

           case WindowHandler::disableCode:
              if (pThis->handlerCount > 1)
                delete ref;  // Dtor called

              // No more handlers?
              if (!--pThis->handlerCount)
              {
                pThis->threadId = 0;  // signals that thread will be dead

                // Detach thread from VM
                pThis->pVm->DetachCurrentThread();
                ref->setDetached(true);
                delete ref;  // Dtor called only now, to allow DetachCurrentThread() first

//                cout << "Thread will die..." << endl;
                ExitThread(0);
   
              }
              break;
         }
      }
      else
      {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }
   }

   // Detach thread from VM
   pThis->pVm->DetachCurrentThread();

   return 0;
}
