// JNIBuffer.cpp

#ifndef __jnibuffer_cpp__
#define __jnibuffer_cpp__

#include "JNIBuffer.h"

// ----------------- Implementation -----------------
// Ctor
template <typename T>
JNIBuffer<T>::JNIBuffer(JNIEnv * env, T jary, jint bufSize)
{
   _myEnv = env;
   _pBoolAry = 0;
   _pCharAry = 0;
   _pByteAry = 0;
   _pShortAry = 0;
   _pIntAry = 0;
   _pLongAry = 0;
   _pFloatAry = 0;
   _pDoubleAry = 0;
   _head = 0;
   _tail = 0;
   _nb = 0;
   _overflow = false;

   // Make NativeHandler reference global, so any JNI methods can is it
   _myAry = (T)(env->NewGlobalRef(jary)); 
   _bufSize = (int)bufSize;
   _arySize = env->GetArrayLength(_myAry);

 //  cout << "typename: " << typeid(T).name() << endl;

   _type = 'E'; // Error
   if (strstr(typeid(T).name(), "_jbooleanArray *") != 0)
      _type = 'Z';
   else if (strstr(typeid(T).name(), "_jcharArray *") != 0)
      _type = 'C';
   else if (strstr(typeid(T).name(), "_jbyteArray *") != 0)
      _type = 'B';
   else if (strstr(typeid(T).name(), "_jshortArray *") != 0)
      _type = 'S';
   else if (strstr(typeid(T).name(), "_jintArray *") != 0)
      _type = 'I';
   else if (strstr(typeid(T).name(), "_jlongArray *") != 0)
      _type = 'J';
   else if (strstr(typeid(T).name(), "_jfloatArray *") != 0)
      _type = 'F';
   else if (strstr(typeid(T).name(), "_jdoubleArray *") != 0)
      _type = 'D';

   if (_type != 'E')
      iniArray();
   else
   {
     char msg[256];
     strcpy(msg, "Illegal type Got: ");
     strcat(msg, typeid(T).name());
     SHOW_ERROR("Fatal error when constructing JNIBuffer", msg);
   }
}

template <typename T>
JNIBuffer<T>::~JNIBuffer()
{
   switch (_type)
   {
      case 'Z':
         delete [] _pBoolAry;
         break;
      case 'C':
         delete [] _pCharAry;
         break;
      case 'B':
         delete [] _pByteAry;
         break;
      case 'S':
         delete [] _pShortAry;
         break;
      case 'I':
         delete [] _pIntAry;
         break;
      case 'J':
         delete [] _pLongAry;
         break;
      case 'F':
         delete [] _pFloatAry;
         break;
      case 'D':
         delete [] _pDoubleAry;
         break;
   }

   _myEnv->DeleteGlobalRef(_myAry);
}

template <>
void JNIBuffer<jbooleanArray>::iniArray()
{
   _pBoolAry = new bool[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pBoolAry[i] = 0;
   _myEnv->SetBooleanArrayRegion(_myAry, 0, _arySize, (jboolean *)_pBoolAry);
}

template <>
void JNIBuffer<jcharArray>::iniArray()
{
   _pCharAry = new wchar_t[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pCharAry[i] = 0;
   _myEnv->SetCharArrayRegion(_myAry, 0, _arySize, (jchar *)_pCharAry);
}

template <>
void JNIBuffer<jbyteArray>::iniArray()
{
   _pByteAry = new signed char[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pByteAry[i] = 0;
   _myEnv->SetByteArrayRegion(_myAry, 0, _arySize, (jbyte *)_pByteAry);
}

template <>
void JNIBuffer<jshortArray>::iniArray()
{
   _pShortAry = new short[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pShortAry[i] = 0;
   _myEnv->SetShortArrayRegion(_myAry, 0, _arySize, (jshort *)_pShortAry);
}

template <>
void JNIBuffer<jintArray>::iniArray()
{
   _pIntAry = new int[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pIntAry[i] = 0;
   _myEnv->SetIntArrayRegion(_myAry, 0, _arySize, (jint *)_pIntAry);
}

template <>
void JNIBuffer<jlongArray>::iniArray()
{
   _pLongAry = new __int64[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pLongAry[i] = 0;
   _myEnv->SetLongArrayRegion(_myAry, 0, _arySize, (jlong *)_pLongAry);
}

template <>
void JNIBuffer<jfloatArray>::iniArray()
{
   _pFloatAry = new float[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pFloatAry[i] = 0;
   _myEnv->SetFloatArrayRegion(_myAry, 0, _arySize, (jfloat *)_pFloatAry);
}

template <>
void JNIBuffer<jdoubleArray>::iniArray()
{
   _pDoubleAry = new double[_bufSize];
   for (int i = 0; i < _bufSize; i++)
      _pDoubleAry[i] = 0;
   _myEnv->SetDoubleArrayRegion(_myAry, 0, _arySize, (jdouble *)_pDoubleAry);
}

template <>
void JNIBuffer<jbooleanArray>::write(bool value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pBoolAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jcharArray>::write(wchar_t value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
    _pCharAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jbyteArray>::write(signed char value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pByteAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jshortArray>::write(short value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pShortAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jintArray>::write(int value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pIntAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jlongArray>::write(__int64 value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pLongAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jfloatArray>::write(float value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pFloatAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <>
void JNIBuffer<jdoubleArray>::write(double value)
{
   if (_nb == _bufSize)
   {
      _overflow = true;
      return;
   }
   _pDoubleAry[_tail] = value;
   _nb++;
   _tail++;
   if (_tail == _bufSize)
      _tail = 0;
}

template <typename T>
int JNIBuffer<T>::read(int count)
{
//   cout << "read, nb: " << _nb <<  endl;
   if (_nb == 0 || count == 0)
      return 0;
   if (count < 0)
      return -1;  // Error

   // We are sure that we can retrieve without wrapping
   if (_head + count <= _bufSize)
   {
      int nbCopy = min(_nb, count);
 //     cout << "retrieve with no wrapping, nbCopy: " << nbCopy << " head: " << _head <<  endl;
      copyBuf(_myAry, 0, nbCopy, _head);
      _head = _head + nbCopy;
      if (_head == _bufSize)
         _head = 0;
      _nb = _nb - nbCopy;
      return nbCopy;
   }

   // We check if there may be need for wrapping
   bool isWrap;
   if (_head + _nb <= _bufSize)
     isWrap = false;
   else
      isWrap = true;

   // Current buffer not wrapped, we can do a simple copy
   if (!isWrap)
   {
      int nbCopy = _nb;
//      cout << "request with wrapping, but buffer not wrapped, nbCopy: " << nbCopy  << " head: " << _head <<  endl;
      copyBuf(_myAry, 0, nbCopy, _head);
      _head = _head + nbCopy;
      if (_head == _bufSize)
         _head = 0;
      _nb = 0;
      return nbCopy;
   }

   // Current buffer is wrapped, we must copy in two blocks
   int nbCopy = min(_nb, count);
//   cout << "wrapping needed, nbCopy: " << nbCopy  << " head: " << _head << endl;
   int nbBlock_1 = _bufSize - _head;
   int nbBlock_2 = nbCopy - nbBlock_1;
   copyBuf(_myAry, 0, nbBlock_1, _head);
   copyBuf(_myAry, nbBlock_1, nbBlock_2, 0);
   _head = nbBlock_2;
   _nb = _nb - nbCopy;
   return nbCopy;
}


template <>
void JNIBuffer<jbooleanArray>::copyBuf(jbooleanArray jary, int start, int count, int offset)
{
   _myEnv->SetBooleanArrayRegion(jary, start, count, (jboolean *)(_pBoolAry + offset));
}

template <>
void JNIBuffer<jcharArray>::copyBuf(jcharArray jary, int start, int count, int offset)
{
   _myEnv->SetCharArrayRegion(jary, start, count, (jchar *)(_pCharAry + offset));
}

template <>
void JNIBuffer<jbyteArray>::copyBuf(jbyteArray jary, int start, int count, int offset)
{
   _myEnv->SetByteArrayRegion(jary, start, count, (jbyte *)(_pByteAry + offset));
}

template <>
void JNIBuffer<jshortArray>::copyBuf(jshortArray jary, int start, int count, int offset)
{
   _myEnv->SetShortArrayRegion(jary, start, count, (jshort *)(_pShortAry + offset));
}

template <>
void JNIBuffer<jintArray>::copyBuf(jintArray jary, int start, int count, int offset)
{
   _myEnv->SetIntArrayRegion(jary, start, count, (jint *)(_pIntAry + offset));

/*
  jint * carr = (jint *)(_myEnv->GetIntArrayElements(jary, 0));
  if (carr = NULL)
    return;
  carr[0] = 7;
  _myEnv->ReleaseIntArrayElements(jary, carr, 0);
*/
  /*
  jint * carr = (jint *)(_myEnv->GetPrimitiveArrayCritical(jary, 0));
  if (carr = NULL)
    return;
  carr[0] = 7;
  _myEnv->ReleasePrimitiveArrayCritical(jary, carr, 0);
  */
}

template <>
void JNIBuffer<jlongArray>::copyBuf(jlongArray jary, int start, int count, int offset)
{
   _myEnv->SetLongArrayRegion(jary, start, count, (jlong *)(_pLongAry + offset));
}

template <>
void JNIBuffer<jfloatArray>::copyBuf(jfloatArray jary, int start, int count, int offset)
{
   _myEnv->SetFloatArrayRegion(jary, start, count, (jfloat *)(_pFloatAry + offset));
}

template <>
void JNIBuffer<jdoubleArray>::copyBuf(jdoubleArray jary, int start, int count, int offset)
{
   _myEnv->SetDoubleArrayRegion(jary, start, count, (jdouble *)(_pDoubleAry + offset));
}

template <typename T>
void JNIBuffer<T>::flush()
{
  _overflow = false;
  _nb = 0;
  _head = _tail = 0;
}

template <typename T>
int JNIBuffer<T>::count()
{
   return _nb;
}

template <typename T>
bool JNIBuffer<T>::isOverflow(bool reset)
{
   bool overflow = _overflow;
   if (reset)
      _overflow = false;
   return overflow;
}

#endif
