// JNIMethod.cpp

#ifndef __jnimethod_cpp__
#define __jnimethod_cpp__

#include <jni.h>
#include <typeinfo.h>
#include "WindowThread.h"
#include "JNIMethod.h"

template <typename T>
JNIMethod<T>::JNIMethod(jobject jObject, const char * name, const char * parSig)
{
  init(jObject, name, parSig);
}

template <typename T>
JNIMethod<T>::JNIMethod(jobject jObject, const char * name)
{
  init(jObject, name, "");
}

template <typename T>
void JNIMethod<T>::init(jobject jObject, const char * name, const char * parSig)
{
  if (jObject == 0)
  {
     SHOW_ERROR("Fatal error when creating JNIMethod", "No object exposed");
  return;
  }

  _env = threadInstance.getEnv();
  _jObject = jObject;
  _pName = new char[strlen(name) + 1];
  strcpy(_pName, name);
  _pParSig = new char[strlen(parSig) + 1];
  strcpy(_pParSig, parSig);
  _type[0] = 0;

  if (strcmp(typeid(T).name(), "void") == 0)
    strcpy(_type, "V");
  if (strcmp(typeid(T).name(), "wchar_t") == 0)
    strcpy(_type, "C");
  if (strcmp(typeid(T).name(), "bool") == 0)
    strcpy(_type, "Z");
  if (strcmp(typeid(T).name(), "signed char") == 0)
    strcpy(_type, "B");
  if (strcmp(typeid(T).name(), "short") == 0)
    strcpy(_type, "S");
  if (strcmp(typeid(T).name(), "int") == 0)
    strcpy(_type, "I");
  if (strcmp(typeid(T).name(), "__int64") == 0)
    strcpy(_type, "J");
  if (strcmp(typeid(T).name(), "float") == 0)
    strcpy(_type, "F");
  if (strcmp(typeid(T).name(), "double") == 0)
    strcpy(_type, "D");
  //  cout << "type: " << typeid(T).name() << endl;

  if (_type[0] == 0)
  {
    char msg[256];
    strcpy(msg, "Illegal type Got: ");
    strcat(msg, typeid(T).name());
    strcat(msg, "\nbut must be one of\n");
    strcat(msg, "bool, signed char, short, int, long, float, double, wchart_t\n");
    SHOW_ERROR("Fatal error when constructing JNIMethod", msg);
    return;
  }
  _mid = getMethodID();
}

template <typename T>
JNIMethod<T>::~JNIMethod()
{
  if (_pName)
    delete [] _pName;
  if (_pParSig)
    delete [] _pParSig;
}

template <typename T>
jmethodID JNIMethod<T>::getMethodID()
{
   char signature[256];
   sprintf(signature, "(%s)%s", _pParSig, _type);
   _mid = _env->GetMethodID(_env->GetObjectClass(_jObject), _pName, signature);

   char msg[128];
   if (!_mid)
   {
     strcpy(msg, "Method ");
     strcat(msg, _pName);
     strcat(msg, " with signature ");
     strcat(msg, signature);
     strcat(msg, "  not found.");
  SHOW_ERROR("Fatal error in JNIMethod::getMethodID()", msg);
     return 0;
   }
   return _mid;
}

template <typename T>
T JNIMethod<T>::call(jvalue * params)
{
   return T(0);
}

// Explicit specializations
template <>
void JNIMethod<void>::call()
{
   _env->CallVoidMethod(_jObject, _mid);
}

template <>
void JNIMethod<void>::call(jvalue * params)
{
   _env->CallVoidMethodA(_jObject, _mid, params);
}

template <>
wchar_t JNIMethod<wchar_t>::call()
{
   return (wchar_t)_env->CallCharMethod(_jObject, _mid);
}

template <>

wchar_t JNIMethod<wchar_t>::call(jvalue * params)
{
   return (wchar_t)_env->CallCharMethodA(_jObject, _mid, params);
}

template <>
bool JNIMethod<bool>::call()
{
 return (_env->CallBooleanMethod(_jObject, _mid) == JNI_TRUE) ? true : false;
}

template <>
bool JNIMethod<bool>::call(jvalue * params)
{
 return (_env->CallBooleanMethodA(_jObject, _mid, params) == JNI_TRUE) ? true : false;
}

template <>
signed char JNIMethod<signed char>::call()
{
   return (signed char)(_env->CallByteMethod(_jObject, _mid));
}

template <>
signed char JNIMethod<signed char>::call(jvalue * params)
{
   return (signed char)(_env->CallByteMethodA(_jObject, _mid, params));
}

template <>
short JNIMethod<short>::call()
{
   return (short)(_env->CallShortMethod(_jObject, _mid));
}

template <>
short JNIMethod<short>::call(jvalue * params)
{
   return (short)(_env->CallShortMethodA(_jObject, _mid, params));
}

template <>
int JNIMethod<int>::call()
{
   return (int)(_env->CallIntMethod(_jObject, _mid));
}

template <>
int JNIMethod<int>::call(jvalue * params)
{
   return (int)(_env->CallIntMethodA(_jObject, _mid, params));
}

template <>
__int64 JNIMethod<__int64>::call()
{
   return (__int64)(_env->CallLongMethod(_jObject, _mid));
}

template <>
__int64 JNIMethod<__int64>::call(jvalue * params)
{
   return (__int64)(_env->CallLongMethodA(_jObject, _mid, params));
}

template <>
float JNIMethod<float>::call()
{
   return (float)(_env->CallFloatMethod(_jObject, _mid));
}

template <>
float JNIMethod<float>::call(jvalue * params)
{
   return (float)(_env->CallFloatMethodA(_jObject, _mid, params));
}

template <>
double JNIMethod<double>::call()
{
   return (double)(_env->CallDoubleMethod(_jObject, _mid));
}

template <>
double JNIMethod<double>::call(jvalue * params)
{
   return (double)(_env->CallDoubleMethodA(_jObject, _mid, params));
}

// End of explicit specializations


#endif
