// JNINativeHandler.cpp
// Native implementation of JNI calls

#ifndef __jninativehandler_cpp__
#define __jninativehandler_cpp__

#include <jni.h>
#include "WindowHandler.h"

#include "ch_aplu_jaw_NativeHandler.h"


// Prototyp, defined in xxx_Handler.cpp
WindowHandler * selectWindowHandler(JNIEnv * env, jobject obj, int select);

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeEnable(JNIEnv * env, jobject obj, jint select)
{
   WindowHandler * wh = selectWindowHandler(env, obj, (int)select);
   if (wh)
      wh->enable(env);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeDestroy(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->disable();
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeExpose(JNIEnv * env, jobject, jlong ref, jobject obj)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->expose(obj);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeShowWindow(JNIEnv * env, jobject, jlong ref, jboolean isVisible, 
                                          jint percent, jint red, jint green, jint blue, jboolean wait)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   if (isVisible == JNI_TRUE)
   {
      if (wait == JNI_TRUE)
         ((WindowHandler *)ref)->showWindow(true, (int)percent, (int)red, (int)green, (int)blue, true);
      else
         ((WindowHandler *)ref)->showWindow(true, (int)percent, (int)red, (int)green, (int)blue, false);
   }
   else
   {
      if (wait == JNI_TRUE)
         ((WindowHandler *)ref)->showWindow(false, (int)percent, (int)red, (int)green, (int)blue, true);
      else
         ((WindowHandler *)ref)->showWindow(false, (int)percent, (int)red, (int)green, (int)blue, false);
   }
}

JNIEXPORT jint JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeInvoke(JNIEnv * env, jobject, jlong ref, jint paramval)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   return (jint)(((WindowHandler *)ref)->invoke((int)paramval));
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3ZI(JNIEnv * env, jobject, jlong ref, jbooleanArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3CI(JNIEnv * env, jobject, jlong ref, jcharArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3BI(JNIEnv * env, jobject, jlong ref, jbyteArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3SI(JNIEnv * env, jobject, jlong ref, jshortArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3II(JNIEnv * env, jobject, jlong ref, jintArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3JI(JNIEnv * env, jobject, jlong ref, jlongArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3FI(JNIEnv * env, jobject, jlong ref, jfloatArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCreateBuf__J_3DI(JNIEnv * env, jobject, jlong ref, jdoubleArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createBuf(jary, size);
}

JNIEXPORT jint JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeReadBuf(JNIEnv * env, jobject, jlong ref, jint count)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   return (jint)(((WindowHandler *)ref)->readBuf((int)count));
}

JNIEXPORT void JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeFlushBuf(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->flushBuf();
}

JNIEXPORT jint JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeCountBuf(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   return (jint)(((WindowHandler *)ref)->countBuf());
}

JNIEXPORT jboolean JNICALL
Java_ch_aplu_jaw_NativeHandler_nativeIsOverflow(JNIEnv * env, jobject, jlong ref, jboolean reset)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   if (reset == JNI_TRUE)
      return (jboolean)(((WindowHandler *)ref)->isOverflow(true));
   else
      return (jboolean)(((WindowHandler *)ref)->isOverflow(false));
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeStartThread(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->startJNIThread();
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeStopThread(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->stopJNIThread();
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeCreateTrayIcon(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createTrayIcon();
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeModifyTrayIcon(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->modifyTrayIcon();
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeDeleteTrayIcon(JNIEnv * env, jobject, jlong ref)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->deleteTrayIcon();
}


JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeCreateIconEventBuf(JNIEnv * env, jobject, jlong ref, jintArray jary, jint size)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->createIconEventBuf(jary, size);
}


JNIEXPORT jint JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeReadIconEventBuf(JNIEnv * env, jobject, jlong ref, jintArray jary, jint count)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   return (jint)(((WindowHandler *)ref)->readIconEventBuf(jary, (int)count));
}

JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeShowBalloonTooltip(JNIEnv * env, jobject, jlong ref, jint timeout, jint flag)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->showBalloonTooltip((int)timeout, (int)flag);
}


JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeSetWindowPosition(JNIEnv * env, jobject, jlong ref, jint xPos, jint yPos)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->setWindowPosition((int)xPos, (int)yPos);
}


JNIEXPORT void JNICALL 
Java_ch_aplu_jaw_NativeHandler_nativeSetWindowSize(JNIEnv * env, jobject, jlong ref, jint width, jint height)
{
   ((WindowHandler*)ref)->getThread()->setEnv(env);
   ((WindowHandler *)ref)->setWindowSize((int)width, (int)height);
}

#endif

