// JNIVar.cpp

#ifndef __jnivar_cpp__
#define __jnivar_cpp__

#include <jni.h>
#include <typeinfo.h>
#include "WindowThread.h"
#include "JNIVar.h"

// disable strcpy warning
#pragma warning(disable : 4996) 

template <typename T>
JNIVar<T>::JNIVar(jobject jObject, const char * name)
{
  if (jObject == 0)
  {
     SHOW_ERROR("Fatal error when creating JNIVar", "No object exposed");
  return;
  }

  _env = threadInstance.getEnv();
  _jObject = jObject;
  _pName = new char[strlen(name) + 1];
  strcpy(_pName, name);
  _type[0] = 0;

  if (strcmp(typeid(T).name(), "wchar_t") == 0)
    strcpy(_type, "C");
  if (strcmp(typeid(T).name(), "bool") == 0)
    strcpy(_type, "Z");
  if (strcmp(typeid(T).name(), "signed char") == 0)
    strcpy(_type, "B");
  if (strcmp(typeid(T).name(), "short") == 0)
    strcpy(_type, "S");
  if (strcmp(typeid(T).name(), "int") == 0)
    strcpy(_type, "I");
  if (strcmp(typeid(T).name(), "__int64") == 0)
    strcpy(_type, "J");
  if (strcmp(typeid(T).name(), "float") == 0)
    strcpy(_type, "F");
  if (strcmp(typeid(T).name(), "double") == 0)
    strcpy(_type, "D");

//    cout << "type: " << typeid(T).name() << endl;

  if (_type[0] == 0)
  {
    char msg[256];
    strcpy(msg, "Illegal type Got: ");
    strcat(msg, typeid(T).name());
    strcat(msg, "\nbut must be one of\n");
    strcat(msg, "bool, signed char, short, int, long, float, double, wchart_t\n");
    SHOW_ERROR("Fatal error when constructing JNIVar", msg);
    return;
  }
  _id = getFieldID();
}

template <typename T>
JNIVar<T>::~JNIVar()
{
  if (_pName)
    delete [] _pName;
}

template <typename T>
jfieldID JNIVar<T>::getFieldID()
{
   char msg[128];
//   cout << "looking for variable " << _pName << " with type: " << _type << endl;
   _id = _env->GetFieldID(_env->GetObjectClass(_jObject), _pName, _type);

   if (!_id)
   {
     strcpy(msg, "Variable ");
     strcat(msg, _pName);
     strcat(msg, " not found.");
  SHOW_ERROR("Fatal error in JNIVar::getFieldID()", msg)
     return 0;
   }
   return _id;
}

template <typename T>
T JNIVar<T>::get()
{
  return T(0);
}

// Explicit specializations
template <>
wchar_t JNIVar<wchar_t>::get()
{
   return (wchar_t)(_env->GetCharField(_jObject, _id));
}

template <>
bool JNIVar<bool>::get()
{
 return (_env->GetBooleanField(_jObject, _id) == JNI_TRUE) ? true : false;
}

template <>
signed char JNIVar<signed char>::get()
{
   return (signed char)(_env->GetByteField(_jObject, _id));
}

template <>
short JNIVar<short>::get()
{
   return (short)(_env->GetShortField(_jObject, _id));
}

template <>
int JNIVar<int>::get()
{
   return (int)(_env->GetIntField(_jObject, _id));
}

template <>
__int64 JNIVar<__int64>::get()
{
   return (__int64)(_env->GetLongField(_jObject, _id));
}

template <>
float JNIVar<float>::get()
{
   return (float)(_env->GetFloatField(_jObject, _id));
}

template <>
double JNIVar<double>::get()
{
   return (double)(_env->GetDoubleField(_jObject, _id));
}

template <>
void JNIVar<wchar_t>::set(wchar_t value)
{
   _env->SetCharField(_jObject, _id, (jchar)value);
}

template <>
void JNIVar<bool>::set(bool value)
{
   _env->SetBooleanField(_jObject, _id, (jboolean)value);
}

template <>
void JNIVar<signed char>::set(signed char value)
{
   _env->SetByteField(_jObject, _id, (jbyte)value);
}

template <>
void JNIVar<short>::set(short value)
{
   _env->SetShortField(_jObject, _id, (jshort)value);
}

template <>
void JNIVar<int>::set(int value)
{
   _env->SetIntField(_jObject, _id, (jint)value);
}

template <>
void JNIVar<__int64>::set(__int64 value)
{
   _env->SetLongField(_jObject, _id, (jlong)value);
}

template <>
void JNIVar<float>::set(float value)
{
   _env->SetFloatField(_jObject, _id, (jfloat)value);
}

template <>
void JNIVar<double>::set(double value)
{
   _env->SetDoubleField(_jObject, _id, (jdouble)value);
}
// End of explicit specializations


#endif

