// WindowList.cpp

#include "WindowList.h"
#include "WindowHandler.h"

WindowListItem::WindowListItem()
{
  hWnd = 0;
  jObject = (jobject)0;
  pWh = 0;
}

WindowListItem::WindowListItem(HWND hwnd, jobject jObj, WindowHandler * ref)
{
  hWnd = hwnd;
  jObject = jObj;
  pWh = ref;
}

// --------------- class WindowList --------------------------
WindowList::WindowList()  // Ctor
{
  pHead = 0;
  pTail = 0;
}

WindowList::~WindowList()  // Dtor
{
//   cout << "calling ~WindowList with size: " << getSize() << endl;
   while ( pHead != 0 )
   {
      ListNode * pNode = pHead;
      delete pNode->pInfo;
      pHead = pHead->pNext;
      delete pNode;
   }
   pTail = 0;
}

void WindowList::insert(HWND hwnd, jobject jObject, WindowHandler * ref)
{
//   cout << "calling insert() with hwnd: " << hwnd << endl;
   ListNode * pNode = new ListNode;   // temperary
   WindowListItem * pItem = new WindowListItem(hwnd, jObject, ref);
   pNode->pInfo = pItem;
   pNode->pNext = 0;
   if (pHead == 0)  // First item to insert
   {
      pHead = pNode;
      pTail = pNode;
   }
   else // Some items already in list
   {
      pTail->pNext = pNode;
      pTail = pNode;
   }
}

jobject WindowList::getObject(HWND hwnd)
{
//   cout << "calling getObject() with hwnd: " << hwnd << endl;
   ListNode * pNode = pHead;
   while ( pNode != 0 )
   {
      if (pNode->pInfo->hWnd == hwnd)
         return pNode->pInfo->jObject;
      pNode = pNode->pNext;
   }
   return (jobject)0; // Not found
}

WindowHandler * WindowList::getHandler(HWND hwnd)
{
//   cout << "calling getHandler() with hwnd: " << hwnd << endl;
   ListNode * pNode = pHead;
   while ( pNode != 0 )
   {
      if (pNode->pInfo->hWnd == hwnd)
         return pNode->pInfo->pWh;
      pNode = pNode->pNext;
   }
   return 0; // Not found
}

WindowHandler * WindowList::getHandler(jobject jObject)
{
//   cout << "calling getHandler() with jobject: " << (int)jObject << endl;
   ListNode * pNode = pHead;
   while ( pNode != 0 )
   {
      if (pNode->pInfo->jObject == jObject)
         return pNode->pInfo->pWh;
      pNode = pNode->pNext;
   }
   return 0; // Not found
}


void WindowList::remove(HWND hwnd)
{
//   cout << "Remove node with hwnd: " << hwnd << endl;
   ListNode * pNode = pHead;
   if (pNode->pInfo->hWnd == hwnd)  // First node to remove
   {
      if (pNode->pNext != 0) // More than one node
      {
         pHead = pNode->pNext;
      }
      else  // Only one node
      {
         pHead = 0;
         pTail = 0;
      }
      delete pNode->pInfo;
      delete pNode;
   }
   else  // Not first node to remove
   {
      while ( pNode->pNext != 0 )
      {
         if (pNode->pNext->pInfo->hWnd  == hwnd)
         {
            ListNode * pDel = pNode->pNext;
            pNode->pNext = pDel->pNext;
            if (pDel == pTail)
               pTail = pNode;
            delete pDel->pInfo;
            delete pDel;
            break;
         }
         pNode = pNode->pNext;
      }
   }
}

int WindowList::getSize()
{
   int size = 0;
   ListNode * pNode = pHead;
   while (pNode != 0)
   {
      size++;
      pNode = pNode->pNext;
   }
   return size;
}



